"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __asyncDelegator = (this && this.__asyncDelegator) || function (o) {
    var i, p;
    return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
    function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
};
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _BlobDataItem_path, _BlobDataItem_start;
Object.defineProperty(exports, "__esModule", { value: true });
exports.fileFromSync = exports.fileFrom = exports.blobFromSync = exports.blobFrom = exports.Blob = exports.File = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const worker_threads_1 = require("worker_threads");
const file_js_1 = __importDefault(require("./file.js"));
exports.File = file_js_1.default;
const index_js_1 = __importDefault(require("./index.js"));
exports.Blob = index_js_1.default;
const { stat } = fs_1.promises;
const DOMException = globalThis.DOMException || (() => {
    const port = new worker_threads_1.MessageChannel().port1;
    const ab = new ArrayBuffer(0);
    try {
        port.postMessage(ab, [ab, ab]);
    }
    catch (err) {
        return err.constructor;
    }
})();
/**
 * @param {string} path filepath on the disk
 * @param {string} [type] mimetype to use
 */
const blobFromSync = (path, type) => fromBlob((0, fs_1.statSync)(path), path, type);
exports.blobFromSync = blobFromSync;
/**
 * @param {string} path filepath on the disk
 * @param {string} [type] mimetype to use
 */
const blobFrom = (path, type) => stat(path).then(stat => fromBlob(stat, path, type));
exports.blobFrom = blobFrom;
/**
 * @param {string} path filepath on the disk
 * @param {string} [type] mimetype to use
 */
const fileFrom = (path, type) => stat(path).then(stat => fromFile(stat, path, type));
exports.fileFrom = fileFrom;
/**
 * @param {string} path filepath on the disk
 * @param {string} [type] mimetype to use
 */
const fileFromSync = (path, type) => fromFile((0, fs_1.statSync)(path), path, type);
exports.fileFromSync = fileFromSync;
// @ts-ignore
const fromBlob = (stat, path, type = '') => new index_js_1.default([new BlobDataItem({
        path,
        size: stat.size,
        lastModified: stat.mtimeMs,
        start: 0
    })], { type });
// @ts-ignore
const fromFile = (stat, path, type = '') => new file_js_1.default([new BlobDataItem({
        path,
        size: stat.size,
        lastModified: stat.mtimeMs,
        start: 0
    })], (0, path_1.basename)(path), { type, lastModified: stat.mtimeMs });
/**
 * This is a blob backed up by a file on the disk
 * with minium requirement. Its wrapped around a Blob as a blobPart
 * so you have no direct access to this.
 *
 * @private
 */
class BlobDataItem {
    constructor(options) {
        _BlobDataItem_path.set(this, void 0);
        _BlobDataItem_start.set(this, void 0);
        __classPrivateFieldSet(this, _BlobDataItem_path, options.path, "f");
        __classPrivateFieldSet(this, _BlobDataItem_start, options.start, "f");
        this.size = options.size;
        this.lastModified = options.lastModified;
    }
    /**
     * Slicing arguments is first validated and formatted
     * to not be out of range by Blob.prototype.slice
     */
    slice(start, end) {
        return new BlobDataItem({
            path: __classPrivateFieldGet(this, _BlobDataItem_path, "f"),
            lastModified: this.lastModified,
            size: end - start,
            start
        });
    }
    stream() {
        return __asyncGenerator(this, arguments, function* stream_1() {
            const { mtimeMs } = yield __await(stat(__classPrivateFieldGet(this, _BlobDataItem_path, "f")));
            if (mtimeMs > this.lastModified) {
                throw new DOMException('The requested file could not be read, typically due to permission problems that have occurred after a reference to a file was acquired.', 'NotReadableError');
            }
            yield __await(yield* __asyncDelegator(__asyncValues((0, fs_1.createReadStream)(__classPrivateFieldGet(this, _BlobDataItem_path, "f"), {
                start: __classPrivateFieldGet(this, _BlobDataItem_start, "f"),
                end: __classPrivateFieldGet(this, _BlobDataItem_start, "f") + this.size - 1
            }))));
        });
    }
    get [(_BlobDataItem_path = new WeakMap(), _BlobDataItem_start = new WeakMap(), Symbol.toStringTag)]() {
        return 'Blob';
    }
}
exports.default = blobFromSync;
